@@ -23,6 +23,7 @@ AWS_SERVERLESS_LAYERVERSION = "AWS::Serverless::LayerVersion"
 AWS_GLUE_JOB = "AWS::Glue::Job"
 AWS_SERVERLESS_STATEMACHINE = "AWS::Serverless::StateMachine"
 AWS_STEPFUNCTIONS_STATEMACHINE = "AWS::StepFunctions::StateMachine"
+AWS_ECR_REPOSITORY = "AWS::ECR::Repository"
 
 METADATA_WITH_LOCAL_PATHS = {AWS_SERVERLESSREPO_APPLICATION: ["LicenseUrl", "ReadmeUrl"]}
 
@@ -50,6 +51,7 @@ RESOURCES_WITH_LOCAL_PATHS = {
 RESOURCES_WITH_IMAGE_COMPONENT = {
     AWS_SERVERLESS_FUNCTION: ["ImageUri"],
     AWS_LAMBDA_FUNCTION: ["Code"],
+    AWS_ECR_REPOSITORY: ["RepositoryName"],
 }
 
 
