@@ -22,6 +22,7 @@ from samcli.lib.package.utils import (
     upload_local_image_artifacts,
     is_s3_protocol_url,
     is_path_value_valid,
+    is_ecr_url,
 )
 
 from samcli.commands._utils.resources import (
@@ -45,6 +46,7 @@ from samcli.commands._utils.resources import (
     METADATA_WITH_LOCAL_PATHS,
     RESOURCES_WITH_LOCAL_PATHS,
     RESOURCES_WITH_IMAGE_COMPONENT,
+    AWS_ECR_REPOSITORY,
 )
 
 from samcli.lib.utils.packagetype import IMAGE, ZIP
@@ -79,6 +81,9 @@ class Resource:
     def do_export(self, resource_id, resource_dict, parent_dir):
         pass
 
+    def delete(self, resource_id, resource_dict):
+        pass
+
 
 class ResourceZip(Resource):
     """
@@ -154,6 +159,33 @@ class ResourceZip(Resource):
             )
         set_value_from_jmespath(resource_dict, self.PROPERTY_NAME, uploaded_url)
 
+    def delete(self, resource_id, resource_dict):
+        """
+        Delete the S3 artifact using S3 url referenced by PROPERTY_NAME
+        """
+        if resource_dict is None:
+            return
+
+        s3_info = self.get_property_value(resource_dict)
+        if s3_info["Key"]:
+            self.uploader.delete_artifact(s3_info["Key"], True)
+
+    def get_property_value(self, resource_dict):
+        """
+        Get the s3 property value for this resource
+        """
+        if resource_dict is None:
+            return {"Bucket": None, "Key": None}
+
+        resource_path = jmespath.search(self.PROPERTY_NAME, resource_dict)
+        # In the case where resource_path is pointing to an intrinsinc
+        # ref function, sam delete will delete the stack but skip the deletion of this
+        # artifact, as deletion of intrinsic ref function artifacts is not supported yet.
+        # TODO: Allow deletion of S3 artifacts with intrinsic ref functions.
+        if resource_path and isinstance(resource_path, str):
+            return self.uploader.parse_s3_url(resource_path)
+        return {"Bucket": None, "Key": None}
+
 
 class ResourceImageDict(Resource):
     """
@@ -197,6 +229,23 @@ class ResourceImageDict(Resource):
         )
         set_value_from_jmespath(resource_dict, self.PROPERTY_NAME, {self.EXPORT_PROPERTY_CODE_KEY: uploaded_url})
 
+    def delete(self, resource_id, resource_dict):
+        """
+        Delete the ECR artifact using ECR url in PROPERTY_NAME referenced by EXPORT_PROPERTY_CODE_KEY
+        """
+        if resource_dict is None:
+            return
+
+        remote_path = resource_dict.get(self.PROPERTY_NAME, {}).get(self.EXPORT_PROPERTY_CODE_KEY)
+        # In the case where remote_path is pointing to an intrinsinc
+        # ref function, sam delete will delete the stack but skip the deletion of this
+        # artifact, as deletion of intrinsic ref function artifacts is not supported yet.
+        # TODO: Allow deletion of ECR artifacts with intrinsic ref functions.
+        if isinstance(remote_path, str) and is_ecr_url(remote_path):
+            self.uploader.delete_artifact(
+                image_uri=remote_path, resource_id=resource_id, property_name=self.PROPERTY_NAME
+            )
+
 
 class ResourceImage(Resource):
     """
@@ -238,6 +287,23 @@ class ResourceImage(Resource):
         )
         set_value_from_jmespath(resource_dict, self.PROPERTY_NAME, uploaded_url)
 
+    def delete(self, resource_id, resource_dict):
+        """
+        Delete the ECR artifact using ECR url referenced by property_name
+        """
+        if resource_dict is None:
+            return
+
+        remote_path = resource_dict.get(self.PROPERTY_NAME)
+        # In the case where remote_path is pointing to an intrinsinc
+        # ref function, sam delete will delete the stack but skip the deletion of this
+        # artifact, as deletion of intrinsic ref function artifacts is not supported yet.
+        # TODO: Allow deletion of ECR artifacts with intrinsic ref functions.
+        if isinstance(remote_path, str) and is_ecr_url(remote_path):
+            self.uploader.delete_artifact(
+                image_uri=remote_path, resource_id=resource_id, property_name=self.PROPERTY_NAME
+            )
+
 
 class ResourceWithS3UrlDict(ResourceZip):
     """
@@ -269,6 +335,37 @@ class ResourceWithS3UrlDict(ResourceZip):
         )
         set_value_from_jmespath(resource_dict, self.PROPERTY_NAME, parsed_url)
 
+    def delete(self, resource_id, resource_dict):
+        """
+        Delete the S3 artifact using S3 url in the dict PROPERTY_NAME
+        using the bucket at BUCKET_NAME_PROPERTY and key at OBJECT_KEY_PROPERTY
+        """
+        if resource_dict is None:
+            return
+
+        s3_info = self.get_property_value(resource_dict)
+        if s3_info["Key"]:
+            self.uploader.delete_artifact(remote_path=s3_info["Key"], is_key=True)
+
+    def get_property_value(self, resource_dict):
+        """
+        Get the s3 property value for this resource
+        """
+        if resource_dict is None:
+            return {"Bucket": None, "Key": None}
+
+        resource_path = resource_dict.get(self.PROPERTY_NAME, {})
+        s3_bucket = resource_path.get(self.BUCKET_NAME_PROPERTY, None)
+
+        key = resource_path.get(self.OBJECT_KEY_PROPERTY, None)
+        # In the case where resource_path is pointing to an intrinsinc
+        # ref function, sam delete will delete the stack but skip the deletion of this
+        # artifact, as deletion of intrinsic ref function artifacts is not supported yet.
+        # TODO: Allow deletion of S3 artifacts with intrinsic ref functions.
+        if isinstance(s3_bucket, str) and isinstance(key, str):
+            return {"Bucket": s3_bucket, "Key": key}
+        return {"Bucket": None, "Key": None}
+
 
 class ServerlessFunctionResource(ResourceZip):
     RESOURCE_TYPE = AWS_SERVERLESS_FUNCTION
@@ -435,6 +532,34 @@ class CloudFormationResourceVersionSchemaHandlerPackage(ResourceZip):
     PROPERTY_NAME = RESOURCES_WITH_LOCAL_PATHS[AWS_CLOUDFORMATION_RESOURCEVERSION][0]
 
 
+class ECRResource(Resource):
+    """
+    Represents CloudFormation resources ECR for deleting the ECR
+    repository with the property name RepositoryName. This class is used
+    only for deleting the repository and not exporting anything.
+    """
+
+    RESOURCE_TYPE = AWS_ECR_REPOSITORY
+    PROPERTY_NAME = RESOURCES_WITH_IMAGE_COMPONENT[RESOURCE_TYPE][0]
+    ARTIFACT_TYPE = ZIP
+    EXPORT_DESTINATION = Destination.ECR
+
+    def delete(self, resource_id, resource_dict):
+        if resource_dict is None:
+            return
+
+        repository_name = self.get_property_value(resource_dict)
+        # TODO: Allow deletion of ECR Repositories with intrinsic ref functions.
+        if repository_name and isinstance(repository_name, str):
+            self.uploader.delete_ecr_repository(physical_id=repository_name)
+
+    def get_property_value(self, resource_dict):
+        if resource_dict is None:
+            return None
+
+        return jmespath.search(self.PROPERTY_NAME, resource_dict)
+
+
 RESOURCES_EXPORT_LIST = [
     ServerlessFunctionResource,
     ServerlessFunctionImageResource,
@@ -456,6 +581,7 @@ RESOURCES_EXPORT_LIST = [
     GlueJobCommandScriptLocationResource,
     CloudFormationModuleVersionModulePackage,
     CloudFormationResourceVersionSchemaHandlerPackage,
+    ECRResource,
 ]
 
 METADATA_EXPORT_LIST = [ServerlessRepoApplicationReadme, ServerlessRepoApplicationLicense]
