@@ -23,7 +23,9 @@ class SamFunctionProvider(SamBaseProvider):
     It may or may not contain a function.
     """
 
-    def __init__(self, stacks: List[Stack], ignore_code_extraction_warnings: bool = False) -> None:
+    def __init__(
+        self, stacks: List[Stack], use_raw_codeuri: bool = False, ignore_code_extraction_warnings: bool = False
+    ) -> None:
         """
         Initialize the class with SAM template data. The SAM template passed to this provider is assumed
         to be valid, normalized and a dictionary. It should be normalized by running all pre-processing
@@ -35,6 +37,8 @@ class SamFunctionProvider(SamBaseProvider):
         You need to explicitly update the class with new template, if necessary.
 
         :param dict stacks: List of stacks functions are extracted from
+        :param bool use_raw_codeuri: Do not resolve adjust core_uri based on the template path, use the raw uri.
+            Note(xinhol): use_raw_codeuri is temporary to fix a bug, and will be removed for a permanent solution.
         :param bool ignore_code_extraction_warnings: Ignores Log warnings
         """
 
@@ -44,7 +48,9 @@ class SamFunctionProvider(SamBaseProvider):
             LOG.debug("%d resources found in the stack %s", len(stack.resources), stack.stack_path)
 
         # Store a map of function full_path to function information for quick reference
-        self.functions = SamFunctionProvider._extract_functions(self.stacks, ignore_code_extraction_warnings)
+        self.functions = SamFunctionProvider._extract_functions(
+            self.stacks, use_raw_codeuri, ignore_code_extraction_warnings
+        )
 
         self._deprecated_runtimes = {"nodejs4.3", "nodejs6.10", "nodejs8.10", "dotnetcore2.0"}
         self._colored = Colored()
@@ -98,12 +104,15 @@ class SamFunctionProvider(SamBaseProvider):
             yield function
 
     @staticmethod
-    def _extract_functions(stacks: List[Stack], ignore_code_extraction_warnings: bool = False) -> Dict[str, Function]:
+    def _extract_functions(
+        stacks: List[Stack], use_raw_codeuri: bool = False, ignore_code_extraction_warnings: bool = False
+    ) -> Dict[str, Function]:
         """
         Extracts and returns function information from the given dictionary of SAM/CloudFormation resources. This
         method supports functions defined with AWS::Serverless::Function and AWS::Lambda::Function
 
         :param stacks: List of SAM/CloudFormation stacks to extract functions from
+        :param bool use_raw_codeuri: Do not resolve adjust core_uri based on the template path, use the raw uri.
         :param bool ignore_code_extraction_warnings: suppress log statements on code extraction from resources.
         :return dict(string : samcli.commands.local.lib.provider.Function): Dictionary of function full_path to the
             Function configuration object
@@ -120,10 +129,34 @@ class SamFunctionProvider(SamBaseProvider):
                 if resource_metadata:
                     resource_properties["Metadata"] = resource_metadata
 
+                if resource_type in [SamFunctionProvider.SERVERLESS_FUNCTION, SamFunctionProvider.LAMBDA_FUNCTION]:
+                    resource_package_type = resource_properties.get("PackageType", ZIP)
+
+                    code_property_key = SamBaseProvider.CODE_PROPERTY_KEYS[resource_type]
+                    image_property_key = SamBaseProvider.IMAGE_PROPERTY_KEYS[resource_type]
+
+                    if resource_package_type == ZIP and SamBaseProvider._is_s3_location(
+                        resource_properties.get(code_property_key)
+                    ):
+
+                        # CodeUri can be a dictionary of S3 Bucket/Key or a S3 URI, neither of which are supported
+                        if not ignore_code_extraction_warnings:
+                            SamFunctionProvider._warn_code_extraction(resource_type, name, code_property_key)
+                        continue
+
+                    if resource_package_type == IMAGE and SamBaseProvider._is_ecr_uri(
+                        resource_properties.get(image_property_key)
+                    ):
+                        # ImageUri can be an ECR uri, which is not supported
+                        if not ignore_code_extraction_warnings:
+                            SamFunctionProvider._warn_imageuri_extraction(resource_type, name, image_property_key)
+                        continue
+
                 if resource_type == SamFunctionProvider.SERVERLESS_FUNCTION:
                     layers = SamFunctionProvider._parse_layer_info(
                         stack,
                         resource_properties.get("Layers", []),
+                        use_raw_codeuri,
                         ignore_code_extraction_warnings=ignore_code_extraction_warnings,
                     )
                     function = SamFunctionProvider._convert_sam_function_resource(
@@ -131,7 +164,7 @@ class SamFunctionProvider(SamBaseProvider):
                         name,
                         resource_properties,
                         layers,
-                        ignore_code_extraction_warnings=ignore_code_extraction_warnings,
+                        use_raw_codeuri,
                     )
                     result[function.full_path] = function
 
@@ -139,10 +172,11 @@ class SamFunctionProvider(SamBaseProvider):
                     layers = SamFunctionProvider._parse_layer_info(
                         stack,
                         resource_properties.get("Layers", []),
+                        use_raw_codeuri,
                         ignore_code_extraction_warnings=ignore_code_extraction_warnings,
                     )
                     function = SamFunctionProvider._convert_lambda_function_resource(
-                        stack, name, resource_properties, layers
+                        stack, name, resource_properties, layers, use_raw_codeuri
                     )
                     result[function.full_path] = function
 
@@ -156,7 +190,7 @@ class SamFunctionProvider(SamBaseProvider):
         name: str,
         resource_properties: Dict,
         layers: List[LayerVersion],
-        ignore_code_extraction_warnings: bool = False,
+        use_raw_codeuri: bool = False,
     ) -> Function:
         """
         Converts a AWS::Serverless::Function resource to a Function configuration usable by the provider.
@@ -184,30 +218,19 @@ class SamFunctionProvider(SamBaseProvider):
                 LOG.debug("Found Serverless function with name='%s' and InlineCode", name)
                 codeuri = None
             else:
-                codeuri = SamFunctionProvider._extract_sam_function_codeuri(
-                    name,
-                    resource_properties,
-                    "CodeUri",
-                    ignore_code_extraction_warnings=ignore_code_extraction_warnings,
-                )
+                codeuri = SamBaseProvider._extract_codeuri(resource_properties, "CodeUri")
                 LOG.debug("Found Serverless function with name='%s' and CodeUri='%s'", name, codeuri)
         elif packagetype == IMAGE:
             imageuri = SamFunctionProvider._extract_sam_function_imageuri(resource_properties, "ImageUri")
             LOG.debug("Found Serverless function with name='%s' and ImageUri='%s'", name, imageuri)
 
         return SamFunctionProvider._build_function_configuration(
-            stack,
-            name,
-            codeuri,
-            resource_properties,
-            layers,
-            inlinecode,
-            imageuri,
+            stack, name, codeuri, resource_properties, layers, inlinecode, imageuri, use_raw_codeuri
         )
 
     @staticmethod
     def _convert_lambda_function_resource(
-        stack: Stack, name: str, resource_properties: Dict, layers: List[LayerVersion]
+        stack: Stack, name: str, resource_properties: Dict, layers: List[LayerVersion], use_raw_codeuri: bool = False
     ) -> Function:
         """
         Converts a AWS::Lambda::Function resource to a Function configuration usable by the provider.
@@ -220,6 +243,8 @@ class SamFunctionProvider(SamBaseProvider):
             Properties of this resource
         layers List(samcli.commands.local.lib.provider.Layer)
             List of the Layer objects created from the template and layer list defined on the function.
+        use_raw_codeuri
+            Do not resolve adjust core_uri based on the template path, use the raw uri.
 
         Returns
         -------
@@ -243,20 +268,14 @@ class SamFunctionProvider(SamBaseProvider):
                 LOG.debug("Found Lambda function with name='%s' and Code ZipFile", name)
                 codeuri = None
             else:
-                codeuri = SamFunctionProvider._extract_lambda_function_code(resource_properties, "Code")
+                codeuri = SamBaseProvider._extract_codeuri(resource_properties, "Code")
                 LOG.debug("Found Lambda function with name='%s' and CodeUri='%s'", name, codeuri)
         elif packagetype == IMAGE:
             imageuri = SamFunctionProvider._extract_lambda_function_imageuri(resource_properties, "Code")
             LOG.debug("Found Lambda function with name='%s' and Imageuri='%s'", name, imageuri)
 
         return SamFunctionProvider._build_function_configuration(
-            stack,
-            name,
-            codeuri,
-            resource_properties,
-            layers,
-            inlinecode,
-            imageuri,
+            stack, name, codeuri, resource_properties, layers, inlinecode, imageuri, use_raw_codeuri
         )
 
     @staticmethod
@@ -268,6 +287,7 @@ class SamFunctionProvider(SamBaseProvider):
         layers: List,
         inlinecode: Optional[str],
         imageuri: Optional[str],
+        use_raw_codeuri: bool = False,
     ) -> Function:
         """
         Builds a Function configuration usable by the provider.
@@ -282,6 +302,8 @@ class SamFunctionProvider(SamBaseProvider):
             Properties of this resource
         layers List(samcli.commands.local.lib.provider.Layer)
             List of the Layer objects created from the template and layer list defined on the function.
+        use_raw_codeuri
+            Do not resolve adjust core_uri based on the template path, use the raw uri.
 
         Returns
         -------
@@ -289,11 +311,20 @@ class SamFunctionProvider(SamBaseProvider):
             Function configuration
         """
         metadata = resource_properties.get("Metadata", None)
-        if metadata and "DockerContext" in metadata:
+        if metadata and "DockerContext" in metadata and not use_raw_codeuri:
+            LOG.debug(
+                "--base-dir is not presented, adjusting uri %s relative to %s",
+                metadata["DockerContext"],
+                stack.location,
+            )
             metadata["DockerContext"] = SamLocalStackProvider.normalize_resource_path(
                 stack.location, metadata["DockerContext"]
             )
 
+        if codeuri and not use_raw_codeuri:
+            LOG.debug("--base-dir is not presented, adjusting uri %s relative to %s", codeuri, stack.location)
+            codeuri = SamLocalStackProvider.normalize_resource_path(stack.location, codeuri)
+
         return Function(
             stack_path=stack.stack_path,
             name=name,
@@ -303,7 +334,7 @@ class SamFunctionProvider(SamBaseProvider):
             memory=resource_properties.get("MemorySize"),
             timeout=resource_properties.get("Timeout"),
             handler=resource_properties.get("Handler"),
-            codeuri=SamLocalStackProvider.normalize_resource_path(stack.location, codeuri) if codeuri else None,
+            codeuri=codeuri,
             imageuri=imageuri if imageuri else resource_properties.get("ImageUri"),
             imageconfig=resource_properties.get("ImageConfig"),
             environment=resource_properties.get("Environment"),
@@ -319,6 +350,7 @@ class SamFunctionProvider(SamBaseProvider):
     def _parse_layer_info(
         stack: Stack,
         list_of_layers: List[Any],
+        use_raw_codeuri: bool = False,
         ignore_code_extraction_warnings: bool = False,
     ) -> List[LayerVersion]:
         """
@@ -331,6 +363,8 @@ class SamFunctionProvider(SamBaseProvider):
         list_of_layers : List[Any]
             List of layers that are defined within the Layers Property on a function,
             layer can be defined as string or Dict, in case customers define it in other types, use "Any" here.
+        use_raw_codeuri : bool
+            Do not resolve adjust core_uri based on the template path, use the raw uri.
         ignore_code_extraction_warnings : bool
             Whether to print warning when codeuri is not a local pth
 
@@ -369,39 +403,58 @@ class SamFunctionProvider(SamBaseProvider):
             # In the list of layers that is defined within a template, you can reference a LayerVersion resource.
             # When running locally, we need to follow that Ref so we can extract the local path to the layer code.
             if isinstance(layer, dict) and layer.get("Ref"):
-                layer_logical_id = cast(str, layer.get("Ref"))
-                layer_resource = stack.resources.get(layer_logical_id)
-                if not layer_resource or layer_resource.get("Type", "") not in (
-                    SamFunctionProvider.SERVERLESS_LAYER,
-                    SamFunctionProvider.LAMBDA_LAYER,
-                ):
-                    raise InvalidLayerReference()
-
-                layer_properties = layer_resource.get("Properties", {})
-                resource_type = layer_resource.get("Type")
-                compatible_runtimes = layer_properties.get("CompatibleRuntimes")
-                codeuri: Optional[str] = None
-
-                if resource_type == SamFunctionProvider.LAMBDA_LAYER:
-                    codeuri = SamFunctionProvider._extract_lambda_function_code(layer_properties, "Content")
-
-                if resource_type == SamFunctionProvider.SERVERLESS_LAYER:
-                    codeuri = SamFunctionProvider._extract_sam_function_codeuri(
-                        layer_logical_id, layer_properties, "ContentUri", ignore_code_extraction_warnings
-                    )
-
-                layers.append(
-                    LayerVersion(
-                        layer_logical_id,
-                        SamLocalStackProvider.normalize_resource_path(stack.location, codeuri) if codeuri else None,
-                        compatible_runtimes,
-                        layer_resource.get("Metadata", None),
-                        stack_path=stack.stack_path,
-                    )
+                found_layer = SamFunctionProvider._locate_layer_from_ref(
+                    stack, layer, use_raw_codeuri, ignore_code_extraction_warnings
+                )
+                if found_layer:
+                    layers.append(found_layer)
+            else:
+                LOG.debug(
+                    'layer "%s" is not recognizable, '
+                    "it might be using intrinsic functions that we don't support yet. Skipping.",
+                    str(layer),
                 )
 
         return layers
 
+    @staticmethod
+    def _locate_layer_from_ref(
+        stack: Stack, layer: Dict, use_raw_codeuri: bool = False, ignore_code_extraction_warnings: bool = False
+    ) -> Optional[LayerVersion]:
+        layer_logical_id = cast(str, layer.get("Ref"))
+        layer_resource = stack.resources.get(layer_logical_id)
+        if not layer_resource or layer_resource.get("Type", "") not in (
+            SamFunctionProvider.SERVERLESS_LAYER,
+            SamFunctionProvider.LAMBDA_LAYER,
+        ):
+            raise InvalidLayerReference()
+
+        layer_properties = layer_resource.get("Properties", {})
+        resource_type = layer_resource.get("Type")
+        compatible_runtimes = layer_properties.get("CompatibleRuntimes")
+        codeuri: Optional[str] = None
+
+        if resource_type in [SamFunctionProvider.LAMBDA_LAYER, SamFunctionProvider.SERVERLESS_LAYER]:
+            code_property_key = SamBaseProvider.CODE_PROPERTY_KEYS[resource_type]
+            if SamBaseProvider._is_s3_location(layer_properties.get(code_property_key)):
+                # Content can be a dictionary of S3 Bucket/Key or a S3 URI, neither of which are supported
+                if not ignore_code_extraction_warnings:
+                    SamFunctionProvider._warn_code_extraction(resource_type, layer_logical_id, code_property_key)
+                return None
+            codeuri = SamBaseProvider._extract_codeuri(layer_properties, code_property_key)
+
+        if codeuri and not use_raw_codeuri:
+            LOG.debug("--base-dir is not presented, adjusting uri %s relative to %s", codeuri, stack.location)
+            codeuri = SamLocalStackProvider.normalize_resource_path(stack.location, codeuri)
+
+        return LayerVersion(
+            layer_logical_id,
+            codeuri,
+            compatible_runtimes,
+            layer_resource.get("Metadata", None),
+            stack_path=stack.stack_path,
+        )
+
     def get_resources_by_stack_path(self, stack_path: str) -> Dict:
         candidates = [stack.resources for stack in self.stacks if stack.stack_path == stack_path]
         if not candidates:
